# Fallible

Fault injection library for stress-testing failure scenarios.

Fallible is a helper library for fault injection.
It is useful to stress-test scenarios that are unlikely to happen on routine tests, such as `malloc` or `fopen` returning NULL.
Combined with Valgrind it asserts that your code is doing the proper clean-up in cases of failure.

Fallible depends on [Valgrind].

[Valgrind]: https://www.valgrind.org


## Usage

```c
// leaky.c
#include <stdlib.h>
#include <string.h>

#ifdef ENABLE_FAULT_INJECTION
#include <fallible/alloc.h>
#endif

int main() {
	char *aaa = malloc(100);
	if (!aaa) {
		return EXIT_FAILURE;
	}
	strcpy(aaa, "a safe use of strcpy");

	char *bbb = malloc(100);
	if (!bbb) {
		// free(aaa);
		return EXIT_FAILURE;
	}
	strcpy(bbb, "not unsafe, but aaa is leaking");

	free(bbb);
	free(aaa);
	return EXIT_SUCCESS;
}
```

Compile with `-DENABLE_FAULT_INJECTION` and run [`fallible-check.1`](fallible-check.1.html):
```shell
$ c99 -DENABLE_FAULT_INJECTION -o leaky leaky.c -lfallible
$ fallible-check ./leaky
Valgrind failed when we did not expect it to:
(...suppressed output...)
# exit status is 1
```


## Installation

Get the latest tarball and install it:

```shell
wget https://git.euandreh.xyz/fallible/snapshot/fallible-main.tar.gz
pax -rzvf fallible-main.tar.gz # or similarly: tar xvf fallible-main.tar.gz
cd fallible-main
make
[sudo] make install
```

The requirements are a C99 compiler on a POSIX.1-2008
environment (`_POSIX_C_SOURCE 200809L`) and Valgrind.


## Documentation

Documentation is available via installed man pages, also available online: [`fallible-check.1`] ([pt][pt.1], [fr][fr.1], [eo][eo.1]) and [`fallible_should_fail.3`] ([pt][pt.3], [fr][fr.3], [eo][eo.3]).

[`fallible-check.1`]: https://euandreh.xyz/fallible/en/fallible-check.1.html
[pt.1]: https://euandreh.xyz/fallible/pt/fallible-check.1.html
[fr.1]: https://euandreh.xyz/fallible/fr/fallible-check.1.html
[eo.1]: https://euandreh.xyz/fallible/eo/fallible-check.1.html
[`fallible_should_fail.3`]: https://euandreh.xyz/fallible/en/fallible_should_fail.3.html
[pt.3]: https://euandreh.xyz/fallible/pt/fallible_should_fail.3.html
[fr.3]: https://euandreh.xyz/fallible/fr/fallible_should_fail.3.html
[eo.3]: https://euandreh.xyz/fallible/eo/fallible_should_fail.3.html


## Contributing

Extra tools used for development are:
- [ShellCheck] for validating scripts;
- [po4a] and [gettext] for i18n and l10n support;
- [pandoc] and Perl for generating the documentation HTML and website.

[ShellCheck]: https://www.shellcheck.net/
[po4a]: https://po4a.org/index.php.en
[gettext]: https://www.gnu.org/software/gettext/
[pandoc]: https://pandoc.org/

For running the extra development-only checks, run:

```shell
$ make dev-check
```

and for generating the documentation HTML and website, run:

```shell
$ make public
```

Send contributions to the [mailing list] via [`git send-email`](https://git-send-email.io/).


## Links

- [homepage](https://euandreh.xyz/fallible/en/)
- [source code](https://git.euandreh.xyz/fallible/)
- [bug tracking](https://euandreh.xyz/fallible/TODOs.html)
- [mailing list]
- [CI logs](https://euandreh.xyz/fallible/ci.html)
- [CHANGELOG](https://euandreh.xyz/fallible/en/CHANGELOG.html)

[mailing list]: https://lists.sr.ht/~euandreh/public-inbox?search=%5Bfallible%5D


## Releases

- [v0.3.0](https://git.euandreh.xyz/fallible/commit/?id=v0.3.0) [fallible-v0.3.0.tar.gz](https://git.euandreh.xyz/fallible/snapshot/fallible-v0.3.0.tar.gz) ([sig](https://git.euandreh.xyz/fallible/snapshot/fallible-v0.3.0.tar.gz.asc)) - 2021-02-25
- [v0.2.1](https://git.euandreh.xyz/fallible/commit/?id=v0.2.1) [fallible-v0.2.1.tar.gz](https://git.euandreh.xyz/fallible/snapshot/fallible-v0.2.1.tar.gz) ([sig](https://git.euandreh.xyz/fallible/snapshot/fallible-v0.2.1.tar.gz.asc)) - 2021-02-22
- [v0.2.0](https://git.euandreh.xyz/fallible/commit/?id=v0.2.0) [fallible-v0.2.0.tar.gz](https://git.euandreh.xyz/fallible/snapshot/fallible-v0.2.0.tar.gz) ([sig](https://git.euandreh.xyz/fallible/snapshot/fallible-v0.2.0.tar.gz.asc)) - 2021-02-21
- [v0.1.0](https://git.euandreh.xyz/fallible/commit/?id=v0.1.0) [fallible-v0.1.0.tar.gz](https://git.euandreh.xyz/fallible/snapshot/fallible-v0.1.0.tar.gz) ([sig](https://git.euandreh.xyz/fallible/snapshot/fallible-v0.1.0.tar.gz.asc)) - 2021-02-16


## License

The code is licensed under [GNU Affero General Public License v3.0 or later][AGPL-3.0-or-later] (AGPL-3.0-or-later).

[AGPL-3.0-or-later]: https://git.euandreh.xyz/fallible/tree/COPYING
